/*
 * JPFMap.java
 *
 * Created on 26. �erven 2007, 14:55
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package pathfinder;

import java.awt.Color;
import java.awt.Component;
import java.awt.geom.Line2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.JFileChooser;

/** Data structure holding map information
 *  
 * @author Petr Buryan
 */
public class JPFMap {
    private int m_cityW=10;
    private int m_cityH=10;
    private Color m_highwayColor=Color.RED;
    private Color m_firstColor=Color.black;
    private Color m_secondColor=Color.green;
    private ArrayList<String[]> m_cityList=null;
    private ArrayList<String[]> m_roadList=null;
    private JMapPanel m_mapPanel = null;
    private Color m_pathColor=Color.blue;
    private Color m_cityColor=Color.orange;
    private Color m_cityColorHighlighted=Color.pink.brighter();
    private int maxX=-100000000;
    private int minX=100000000;
    private int maxY=-100000000;
    private int minY=100000000;
    private int m_margin=10;
    
    
    /**
     *
     * @return number of cities in the map
     */
    public int getCityCount(){
        return this.m_cityList.size();
    }

    /** Creates a new instance of JPFMap */
    public JPFMap() {
        Component frame = null;        
        String curDir = System.getProperty("user.dir");
        JFileChooser fc = new JFileChooser(new File(curDir));      
        
        m_cityList=new ArrayList();
        m_roadList=new ArrayList();
        fc.showOpenDialog(frame);
        File selFile = fc.getSelectedFile();        
            if (selFile==null){return;}        
        loadMapFromFile(selFile);
    }
    
    /** Creates a new instance of JPFMap 
     *@param selFile csv file with map data
     */    
    public JPFMap (File selFile) {        
        // Read comma seperated file
        m_cityList = new ArrayList();
        m_roadList = new ArrayList();
        loadMapFromFile(selFile);
    }    

    /**Loads map from given csv file
     * @param selFile - csv file containing the map
     */
    
    public void loadMapFromFile(File selFile){
        BufferedReader csvReader = null;
        try {
            csvReader = new BufferedReader(new FileReader(selFile));
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        }
         int fieldCount = 0;
         int lineCount=0;
         String[] csvFields = null;
         String[] header=null;
         String curLine;
         String patternStr=",";
         boolean loadCities=true;
        try {
            while((curLine = csvReader.readLine()) != null)
            {
                csvFields = curLine.split(patternStr);
                fieldCount = csvFields.length;
                if (fieldCount==1){
                    loadCities=false;                    
                } else{
                
                lineCount++;
                if (loadCities){
                    m_cityList.add(deblank(csvFields));
                    processCoords(csvFields);
                } else {
                    m_roadList.add(deblank(csvFields));
                    }
                }
            }
        } catch (IOException ex) { 
            ex.printStackTrace();}            
        try { csvReader.close(); 
        } catch (IOException ex) {
            ex.printStackTrace();
        }         
    }
    
    private String[] deblank(String[] s){
        for(int i=0;i<s.length;i++){
            s[i]=s[i].replace(" ","");
        }
        return s;
    }
    
    /** draws the map
     * @param width - width of the output
     * @param height - height of the output
     */
    public JMapPanel drawMap(int width, int height){        
        //m_mapPanel = new JMapPanel(width, height, m_cityList, m_roadList);
        m_mapPanel = new JMapPanel(width, height, this);
        return m_mapPanel;
    }
    
    /** Sets width of the city symbol
     * @param w - width of the symbol
     */
    public void setCityWidth(int w){
        m_cityW=w;
        m_mapPanel.setCityWidth(w);
    }
    
    /** Sets height of the city symbol
     * @param h - height of the symbol
     */    
    public void setCityHeight(int h){
        m_cityH=h;
        m_mapPanel.setCityHeight(h);
    }    
    
    /** Sets colors of the m_roadList
     * @param Hi
     * @param Mid
     * @param Lo - three colors of three road types
     */    
    public void setRoadColors(Color Hi, Color Mid, Color Lo){
        m_highwayColor=Hi;
        m_firstColor=Mid;
        m_secondColor=Lo;
        m_mapPanel.setRoadColors(Hi,Mid,Lo);
    }        

    /** Sets the path color used for drawing paths in the map
     */
    public void setPathColor(Color P){
        m_pathColor=P;
        m_mapPanel.setPathColor(P);
    }  
    
    /** gets path highlight color
     *@return highlight color
     */
    public Color getPathColor(){
        return m_pathColor;
    }
    
    /** sets color of the city symbol
     *@param col color
     */
    public void setCityColor(Color col){
        m_cityColor=col;
        m_mapPanel.setCityColor(col);
    }             
    
    /** returns color of the city symbol
     *@return city color
     */
    public Color getCityColor(){
        return m_cityColor;
    }

    /** returns list of m_cityList in the map
     *@return list of city ids
     */
    public ArrayList<String[]> getCityList() {
        return m_cityList;
    }
    
    /** returns list of m_roadList in the map
     *@return list of road ids
     */
    public ArrayList<String[]> getRoadList() {
        return m_roadList;
    }
    
    /** draws path (=line) between 2 m_cityList using global  path color
     *@param city1 id of 1st city 
     *@param city2 id of 2nd city
     */
    public void drawPath(int city1, int city2){
        if (m_mapPanel==null){return;}
        m_mapPanel.drawPath(city1,city2);
    }
    
    /** draws path (=line) between 2 m_cityList using given path color
     *@param city1 id of 1st city 
     *@param city2 id of 2nd city
     *@param col - highlight color
     */    
    public void drawPath(int city1, int city2, Color col){
        if (m_mapPanel==null){return;}
        Color c= getPathColor();        
        m_mapPanel.drawPath(city1,city2,col);
        m_mapPanel.drawCity(city1);
        m_mapPanel.drawCity(city2);
    }    
    
    /** highlights given array of m_cityList
     * @param cities - array of cities (defined as integers)
     */
    public void highlightCities(int[] cities){
        for(int i=0;i<cities.length;i++){
            m_mapPanel.setCityColor(m_cityColorHighlighted);
            m_mapPanel.drawCity(cities[i]);
            m_mapPanel.setCityColor(m_cityColor);
        }
    }

    /** highlights given array of m_cityList
     * @param cities - array of cities (defined as integers)
     * @param col - highlighting color
     */    
    public void highlightCities(int[] cities, Color col){
        Color c = getCityColor();
        setCityColor(col);
        for(int i=0;i<cities.length;i++){            
            m_mapPanel.drawCity(cities[i]);
        }
        setCityColor(c);
    }    

    /////////////////////////////////////////////
    /** returns city coordinates
     * @param city - id of the city
     * @return coordinates of the city defined as array of 2 integers
     */    
    public int[] getCityCoords(int city){
        //returns city coordinates
    int[] c=new int[2];
        for(int i=0;i<m_cityList.size(); i++){
             String[] s=(String[]) m_cityList.get(i);
             int id = Integer.valueOf(s[0]);
             if(id==city){
                c[0] = Integer.valueOf(s[1]);
                c[1] = Integer.valueOf(s[2]); 
                return c;
             }
        }
    return null;
    }        

    /** returns array of city neighbours
     * @param city - id of the city
     * @return coordinates of the city defined as array of 2 integers
     */
    public int[] getCityNeighbours(int city){
        int cntr=0;
        for(int i=0;i<m_roadList.size();i++){
            String[] s=(String[]) m_roadList.get(i);
            int speed=Integer.valueOf(s[2]);
            if(speed<=0)
                continue;
            int c1=Integer.valueOf(s[0]);
            int c2=Integer.valueOf(s[1]);             
            if((c1==city)||(c2==city)){cntr++;}
        }
        int[] ns = new int[cntr];
        cntr=0;
        for(int i=0;i<m_roadList.size();i++){
            String[] s=(String[]) m_roadList.get(i);
            int speed=Integer.valueOf(s[2]);
            if(speed<=0)
                continue;
            int c1=Integer.valueOf(s[0]);
            int c2=Integer.valueOf(s[1]);             

            if(c1==city){ns[cntr]=c2;cntr++;}
            if(c2==city){ns[cntr]=c1;cntr++;}
        }        
        return ns;
    }
    
    /** returns double distance of given two m_cityList
     *@param city1 id of 1st city 
     *@param city2 id of 2nd city
     * @return (double) distance
     */ 
    public double getCityDistance(int city1, int city2)    {
        int[] c1=getCityCoords(city1);
        int[] c2=getCityCoords(city2);
        double x1=(double) c1[0];
        double y1=(double) c1[1];
        double x2=(double) c2[0];
        double y2=(double) c2[1];
        double d=Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
        return d;
    }

    /** Returns id of the city that is nearest to given position
     * @param x x coordinate
     * @param y y coordinate
     * @return id of the nearest city or -1 if there is no such city
     */
    public int getNearestCity(float x, float y){
        float minD=Float.MAX_VALUE;
        int cityId=-1;
        for(String[]c:m_cityList){
            float cityX=Float.parseFloat(c[1]);
            float cityY=Float.parseFloat(c[2]);
            float d=(x-cityX)*(x-cityX)+(y-cityY)*(y-cityY);
            if(d<minD){
                minD=d;
                cityId=Integer.parseInt(c[0]);
            }
        }
        return cityId;
    }
    
    /** Returns id of the cities that are connected by road nearest to given position
     * @param x x coordinate
     * @param y y coordinate
     * @return id of the road cities or null if there is no such city
     */
    public int[] getNearestRoad(float x, float y, float minDist){
        double minD=Double.MAX_VALUE;
        int roadId=1;
        int roadCntr=0;
        for(String[]c:m_roadList){
            int speed=Integer.parseInt(c[2]);
            if(speed<=0){
                roadCntr++;
                continue;
            }
            int city1=Integer.parseInt(c[0]);
            int city2=Integer.parseInt(c[1]);
            int[]cc1=getCityCoords(city1);
            int[]cc2=getCityCoords(city2);
            Line2D road = new Line2D.Float(cc1[0],cc1[1],cc2[0],cc2[1]);
            double d=road.ptSegDist(x, y);
//System.out.println(city1+"-"+city2+": "+d);
            if(d<minD){
                minD=d;
                roadId=roadCntr;
            }
            roadCntr++;
        }
        if(minD>minDist)
            return null;
        String[]roadCities=m_roadList.get(roadId);
        return new int[]{Integer.parseInt(roadCities[0]), Integer.parseInt(roadCities[1]), roadId};
    }


    /** Blocks road roadId
     *
     * @param roadId
     */
    public void blockRoad(int roadId) {
        String[] road=m_roadList.get(roadId);//m_roadList.get(roadId)[2]
        road[2]=( (-1)*Math.abs(Integer.parseInt(road[2])) )+"";
        m_roadList.set(roadId, road);
    }

    /**
     *  Removes road blocks
     */
    public void unblockRoads() {
        for(int roadId=0;roadId<m_roadList.size();roadId++){
            String[] road=m_roadList.get(roadId);
            road[2]=( Math.abs(Integer.parseInt(road[2])) )+"";
            m_roadList.set(roadId, road);
        }
    }


    /** returns road type (=max. allowed speed) between two m_cityList
     *@param city1 id of 1st city 
     *@param city2 id of 2nd city
     * @return (integer) max speed of the road between given 2 m_cityList
     */
    public int getRoadSpeed(int city1, int city2)    {
            for(int i=0;i<m_roadList.size();i++){
                String[] s=(String[]) m_roadList.get(i);
                int c1=Integer.valueOf(s[0]);
                int c2=Integer.valueOf(s[1]);
                int spd=Integer.valueOf(s[2]);
                if((c1==city1)&&(c2==city2))
                    {return spd;}
                if((c2==city1)&&(c1==city2))
                    {return spd;}
            }
            return 0;
    }
    /** a function utilised for drawing purposes*/
    protected void processCoords(String[] csvFields) {
           Double x= Double.valueOf(csvFields[1]);
           Double y = Double.valueOf(csvFields[2]);         
           if(x>maxX) maxX=Integer.valueOf((int)Math.round(x));
           if(x<minX) minX=Integer.valueOf((int)Math.round(x));
           if(y>maxY) maxY=Integer.valueOf((int)Math.round(y));
           if(y<minY) minY=Integer.valueOf((int)Math.round(y));
    }
    /** a function utilised for drawing purposes*/
    protected double[] remapCoordsToDouble(double[] coords, double windowW, double windowH){
        coords[0]=(coords[0]-minX)*windowW/(maxX-minX+m_margin);
        coords[1]=(coords[1]-minY)*windowH/(maxY-minY+m_margin)+m_margin;
        return coords;
    }
    /** a function utilised for drawing purposes*/
    protected int[] remapCoords(double[] coords, double windowW, double windowH){        
        int[] c=new int[2];
        c[0]=Integer.valueOf((int)Math.round((coords[0]-minX)*windowW/(maxX-minX+2*m_margin)))+m_margin;
        c[1]=Integer.valueOf((int)Math.round((coords[1]-minY)*windowH/(maxY-minY+2*m_margin)))+m_margin;
        return c;
    }

    /** a function utilised for drawing purposes*/
    protected int[] remapCoords(int[] coords, double windowW, double windowH){
        //add +2*m_margin to fit into the window
        int[] c=new int[2];
        c[0]=Integer.valueOf((int)Math.round((coords[0]-minX)*windowW/(maxX-minX+2*m_margin)))+m_margin;
        c[1]=Integer.valueOf((int)Math.round((coords[1]-minY)*windowH/(maxY-minY+2*m_margin)))+m_margin;
        return c;
    }
    
    /** a function utilised for drawing purposes*/
    protected int[] remapCoordsBack(double[] windowCoords, double windowW, double windowH){
        int[] c=new int[2];
        c[0]=(int) ((windowCoords[0] - m_margin) * (maxX - minX + 2 * m_margin) / (windowW * 1f) + minX);
        c[1]= (int) ((windowCoords[1] - m_margin) * (maxY - minY + 2 * m_margin) / (windowH * 1f) + minY);
        return c;
    }

    /** a function utilised for drawing purposes to map from panel to JPFMap
     */
    public int[] remapCoordsBack(int[] windowCoords, double windowW, double windowH) {
        int[] c=new int[2];
        c[0]=(int) ((windowCoords[0] - m_margin) * (maxX - minX + 2 * m_margin) / (windowW * 1f) + minX);
        c[1]= (int) ((windowCoords[1] - m_margin) * (maxY - minY + 2 * m_margin) / (windowH * 1f) + minY);
        return c;
    }

//    class JPFRoad{
//        boolean m_blocked=false;
//        int m_city1, m_city2, m_speed;
//
//        public JPFRoad(String[]roadDef){
//            m_city1=Integer.parseInt(roadDef[0]);
//            m_city2=Integer.parseInt(roadDef[1]);
//            m_speed=Integer.parseInt(roadDef[2]);
//        }
//
//        public JPFRoad(int cityId1, int cityId2, int speed){
//            m_city1=cityId1;
//            m_city2=cityId2;
//            m_speed=speed;
//        }
//
//        public void unblock(){
//            m_blocked=false;
//        }
//
//        public void block(){
//            m_blocked=true;
//        }
//    }
}
