/*
 * JPathFinder.java
 *
 * Created on 26. �erven 2007, 23:50
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package pathfinder;

import gui.MapFrame;
import javax.swing.JTextArea;

/**
 *
 * @author Petr Buryan
 */
public abstract class JPathFinder {
    /**
     * the place where some info can be written
     */
    private JTextArea m_infoTextArea;
   
    /** Creates a new instance of JPathFinder */
    public JPathFinder() {
    }
    

    /** implements the depth first search (performDFS) path search
     * - searches a road from given start to target city
     *@param start id of starting city
     *@param stop id of target city
     *@param map the map to be searched
     */    
    public abstract void performDFS(int start, int stop, JPFMap map);
    

    /** implements the breadth first search (performBFS) path search
     * - searches a road from given start to target city
     *@param start id of starting city
     *@param stop id of target city
     *@param map the map to be searched
     */
    public abstract void performBFS(int start, int stop, JPFMap map);


    /** implements the A* path search 
     * - searches shortest road from given start to target city
     *@param start id of starting city
     *@param stop id of target city
     *@param map the map to be searched
     */    
    public abstract void searchForShortestPath(int start, int stop, JPFMap map);
      //inicializace

    /** implements the A* path search 
     * - searches fastest road from given start to target city
     *@param start id of starting city
     *@param stop id of target city
     *@param map the map to be searched
     */ 
    public abstract void searchForShortestTime(int start, int stop, JPFMap map);

    /**
     * Shows the searcher's GUI
     */
    public void showGUI() {
        final JPathFinder pf=this;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new MapFrame(pf).setVisible(true);
            }
        });
    }

    /**Shows the searcher's GUI and loads the map from given file
     *
     * @param mapFileName map file
     */
    public void showGUI(final String mapFileName) {
        final JPathFinder pf=this;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new MapFrame(pf, mapFileName).setVisible(true);
            }
        });
    }

    /** Sets the place where the info can be written
     *
     * @param infoTextArea
     */
    public void setInformationArea(JTextArea infoTextArea) {
        m_infoTextArea=infoTextArea;
    }

    /**
     * Clears the info box
     */
    public void clearInfoArea(){
        if(m_infoTextArea==null)
            return;
        m_infoTextArea.setText("");
    }

    /** Writes the message to the information window of the GUI
     *
     * @param message
     */
    public void inform(String message){
        if(m_infoTextArea==null)
            return;
        m_infoTextArea.append(message);
        m_infoTextArea.append("\n");
        m_infoTextArea.setCaretPosition(m_infoTextArea.getText().length());
    }

}
